import os
import sys

import cv2
import numpy as np
from keras import Sequential
from keras.layers import Conv2D, MaxPooling2D, Dropout, Flatten, Dense

from .classMapper import TRAFFIC_SIGNS_NAMES
from .csvReader import *

image_resolution = (50, 50)


def prepare_train_data(path):
    images = []
    labels = []
    classCount = len(os.listdir(path))
    for x in range(0, classCount):
        showProgress("Loading train sources: ", x, classCount)
        imageDir = path + "/" + str(x)
        imageList = os.listdir(imageDir)

        for y in imageList:
            currentImg = cv2.imread(imageDir + "/" + y)
            currentImg = cv2.resize(currentImg, image_resolution)
            images.append(currentImg)
            labels.append(x)
    print("")

    images = np.array(images)
    labels = np.array(labels)

    return images, labels


def prepare_test_images(path, file):
    images = []
    imageList = os.listdir(path + "Test")
    labels, paths = get_labels_and_paths(file)

    current = 0
    for imagePath in paths:
        showProgress(str("Loading test sources: "), current, len(imageList))

        currentImg = cv2.imread(path + "/" + imagePath)
        currentImg = cv2.resize(currentImg, image_resolution)

        images.append(currentImg)
        current += 1

    print("")
    images = np.array(images)

    return images, labels


def process_image(img):
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)  # greyscale
    img = cv2.equalizeHist(img)  # contrast
    img = img / 255  # converting to <0,1>
    return img


def reshape(npArray):
    return npArray.reshape(npArray.shape[0], npArray.shape[1], npArray.shape[2], 1)


def create_model():
    kernel_size1 = (5, 5)
    kernel_size2 = (3, 3)
    pool_size = (2, 2)
    filters = 32
    img_shape = (50, 50, 1)
    node_count = 256
    class_count = 43

    model = Sequential()

    model.add(Conv2D(filters=filters, kernel_size=kernel_size1, activation="relu", input_shape=img_shape))
    model.add(Conv2D(filters=filters, kernel_size=kernel_size1, activation="relu"))
    model.add(MaxPooling2D(pool_size=pool_size))
    model.add(Dropout(rate=0.25))
    model.add((Conv2D(filters=2 * filters, kernel_size=kernel_size2, activation="relu")))
    model.add((MaxPooling2D(pool_size=pool_size)))
    model.add(Dropout(rate=0.25))
    model.add(Flatten())
    model.add(Dense(node_count, activation="relu"))
    model.add(Dropout(rate=0.40))
    model.add(Dense(class_count, activation="softmax"))

    model.compile(loss="categorical_crossentropy", optimizer="adam", metrics=["accuracy"])

    return model


def compose_text(classId, probability):
    return TRAFFIC_SIGNS_NAMES[classId] + " (" + str(
        round(probability * 100, 2)) + "%)"


def showProgress(text, current, len):
    sys.stdout.write("\r" + text + str(current + 1) + "/" + str(len))
    sys.stdout.flush()


def getImagesForTesting(imageClass):
    return [imageClass]
