import os

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
import pickle

import cv2
import numpy as np

from helpers.utils import process_image, compose_text

# Settings
width = 1280
height = 720
threshold = 0.85
font = cv2.FONT_HERSHEY_SIMPLEX
fontScale = 1
position = (10, height - 20)
thickness = 1
color = (0, 0, 255)
#

# Obtaining web camera
cap = cv2.VideoCapture(0)
# cap.set(widthIndex, width)
cap.set(3, width)
# cap.set(heightIndex, height)
cap.set(4, height)

# Read data generated by training model
generatedOutput = open("modelOutput/output.p", "rb")
model = pickle.load(generatedOutput)

while True:
    success, imgOriginal = cap.read()
    img = np.asarray(imgOriginal)
    img = cv2.resize(img, (50, 50))
    img = process_image(img)
    img = img.reshape(1, 50, 50, 1)

    # Predicted classID
    classId = int(model.predict_classes(img))
    prediction = model.predict(img)
    # Accuracy of prediction
    probability = np.amax(prediction)
    textToRender = compose_text(classId, probability)

    if probability > threshold:
        cv2.putText(imgOriginal, textToRender, position, font, fontScale, color, thickness, cv2.LINE_AA)
    # Rendering original non-processed captured image
    cv2.imshow("Original image", imgOriginal)

    # "q" key for exit
    if cv2.waitKey(1) & 0xFF == ord("q"):
        break