import os

from keras.callbacks import EarlyStopping, ReduceLROnPlateau

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
import matplotlib.pyplot as plt
from helpers.utils import *
from keras.utils.np_utils import to_categorical
import pickle
from sklearn.model_selection import train_test_split

# Dataset from: https://www.kaggle.com/meowmeowmeowmeowmeow/gtsrb-german-traffic-sign

# Settings
dataTrainPath = "../data/Train"
dataTestPath = "../data/"

batchSize = 64
epochs = 20
#
classCount = len(os.listdir(dataTrainPath))

print("Total class count: ", len(os.listdir(dataTrainPath)))

# Preparing data
train_images, train_labels = prepare_train_data(dataTrainPath)
test_images, test_labels = prepare_test_images(dataTestPath, "../data/Test.csv")

# Splitting for train and validation
X_train, X_valid, y_train, y_valid = train_test_split(train_images, train_labels, test_size=0.2)

# Getting image count for each class
sampleCount = []
for x in range(0, classCount):
    sampleCount.append(len(np.where(train_labels == x)[0]))

# Chart with images per class statistics
plt.figure(figsize=(10, 5))
plt.bar(range(0, classCount), sampleCount)
plt.xlabel("Class ID")
plt.ylabel("Number of Images for each Class")
plt.show()

# Processing each image
X_train = np.array(list(map(process_image, X_train)))
X_valid = np.array(list(map(process_image, X_valid)))
test_images = np.array(list(map(process_image, test_images)))

# Adding 4th (1) parameter to shape (color parameter)
X_train = reshape(X_train)
X_valid = reshape(X_valid)
test_images = reshape(test_images)

# Converting to binary matrix
y_train = to_categorical(y_train, classCount)
y_valid = to_categorical(y_valid, classCount)
test_labels = to_categorical(test_labels, classCount)

# Creating training model
model = create_model()
print(model.summary())

# Callbacks
early_stopping = EarlyStopping(monitor="val_loss", min_delta=0, patience=3, verbose=1)
reduce_lr = ReduceLROnPlateau(monitor="val_loss", factor=0.2, patience=5, verbose=1, min_lr=0.001)

history = model.fit(X_train, y_train, epochs=epochs, batch_size=batchSize, validation_data=(X_valid, y_valid),
                    callbacks=[early_stopping, reduce_lr])

# Plotting accuracy chart
plt.figure(0)
plt.plot(history.history['accuracy'], label="Training accuracy")
plt.plot(history.history['val_accuracy'], label="val accuracy")
plt.title("Accuracy Graph")
plt.xlabel("epochs")
plt.ylabel("accuracy (0,1)")
plt.legend()

# Plotting loss chart
plt.figure(1)
plt.plot(history.history['loss'], label="training loss")
plt.plot(history.history['val_loss'], label="val loss")
plt.title("Loss Graph")
plt.xlabel("epochs")
plt.ylabel("Loss")
plt.legend()
plt.show()

score = model.evaluate(test_images, test_labels, verbose=0)
print("Test loss = ", score[0])
print("Test accuracy = ", score[1])

# Saving generated output by model into file
output = open("modelOutput/output.p", "wb")
pickle.dump(model, output)
output.close()
